--
-- RailAttachment
-- Specialization for  Rail Attachment
--
-- @author  MAurUS	
-- @date  07/04/13

-- Copyright  MAurUS, www.stefanmaurus.de

Vehicle.registerJointType("railAttachment");
Vehicle.registerJointType("fastCoupling");

RailAttachment = {};

function RailAttachment.prerequisitesPresent(specializations)
    return true;
end;

function RailAttachment:load(xmlFile)	
	self.setLowTrailerAttachmentState = SpecializationUtil.callSpecializationsFunction("setLowTrailerAttachmentState"); 	
    self.lowTrailerAttachment = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.lowTrailerAttachment#index")); 
	self.lowTrailerAttachmentState = 1;
	self.lowTrailerIsAttached = false;
end;

function RailAttachment:delete()
end;

function RailAttachment:mouseEvent(posX, posY, isDown, isUp, button)
end;

function RailAttachment:keyEvent(unicode, sym, modifier, isDown)
end;

function RailAttachment:readStream(streamId, connection)
	self:setLowTrailerAttachmentState(streamReadInt8(streamId), true); 
end;

function RailAttachment:writeStream(streamId, connection) 
	streamWriteInt8(streamId, self.lowTrailerAttachmentState); 
end;

function RailAttachment:setLowTrailerAttachmentState(int, noEventSend)
	self.lowTrailerAttachmentState = int;
	setLowTrailerAttachmentStateEvent.sendEvent(self, int, noEventSend);
end;

function RailAttachment:detachImplement()
	self.lowTrailerIsAttached = false;
end;

function RailAttachment:attachImplement(implement)
	local jointType = implement.object.attacherJoint.jointType;
	if jointType == Vehicle.JOINTTYPE_TRAILER or jointType == Vehicle.JOINTTYPE_TRAILERLOW then
		self.lowTrailerIsAttached = true;
	end;	
end;

function RailAttachment:update(dt)
	if self:getIsActiveForInput() then
		if InputBinding.hasEvent(InputBinding.RAILATTACHMENT_LOWATTACHER) and not self.lowTrailerIsAttached then
			if self.lowTrailerAttachmentState == 0 then
				self:setLowTrailerAttachmentState(1);
			elseif self.lowTrailerAttachmentState == 1 then
				self:setLowTrailerAttachmentState(0);
			end;
		end;	
	end;
end;

function RailAttachment:updateTick(dt)
	if self.lowTrailerIsAttached and self.lowTrailerAttachmentState == 0 then
		self:setLowTrailerAttachmentState(1);		
	end;
	if self.lowTrailerAttachmentState == 0 then
		setVisibility(self.lowTrailerAttachment, false);		
	elseif self.lowTrailerAttachmentState == 1 then
		setVisibility(self.lowTrailerAttachment, true);		
	end;
end;

function RailAttachment:onAttach(attacherVehicle, jointDescIndex)
	self.attachedVehicleJointDescIndex = jointDescIndex;
end;

function RailAttachment:onDetach()	
end;

function RailAttachment:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	local lowTrailerAttachmentState = getXMLInt(xmlFile, key .. "#lowTrailerAttachment");
	if lowTrailerAttachmentState ~= nil then
		self:setLowTrailerAttachmentState(lowTrailerAttachmentState);		
	end;
	
	return BaseMission.VEHICLE_LOAD_OK;
end;
 
function RailAttachment:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'lowTrailerAttachment="' .. self.lowTrailerAttachmentState .. '" ';
	return attributes;
end;

function RailAttachment:draw()
	if self.lowTrailerAttachmentState == 0 then
		g_currentMission:addHelpButtonText(g_i18n:getText("RAILATTACHMENT_LOWATTACHER0"), InputBinding.RAILATTACHMENT_LOWATTACHER);		
	elseif self.lowTrailerAttachmentState == 1 then
		g_currentMission:addHelpButtonText(g_i18n:getText("RAILATTACHMENT_LOWATTACHER1"), InputBinding.RAILATTACHMENT_LOWATTACHER);	
	end;	
end;

setLowTrailerAttachmentStateEvent = {};
setLowTrailerAttachmentStateEvent_mt = Class(setLowTrailerAttachmentStateEvent, Event);

InitEventClass(setLowTrailerAttachmentStateEvent, "setLowTrailerAttachmentStateEvent");

function setLowTrailerAttachmentStateEvent:emptyNew()
    local self = Event:new(setLowTrailerAttachmentStateEvent_mt);
    self.className="setLowTrailerAttachmentStateEvent";
    return self;
end;

function setLowTrailerAttachmentStateEvent:new(vehicle, lowTrailerAttachmentState)
    local self = setLowTrailerAttachmentStateEvent:emptyNew()
    self.vehicle = vehicle;
	self.lowTrailerAttachmentState = lowTrailerAttachmentState;
    return self;
end;

function setLowTrailerAttachmentStateEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
    self.vehicle = networkGetObject(id);

	self.lowTrailerAttachmentState = streamReadInt8(streamId);
	self.vehicle:setLowTrailerAttachmentState(self.lowTrailerAttachmentState, true);
	
	if not connection:getIsServer() then
        g_server:broadcastEvent(setLowTrailerAttachmentStateEvent:new(self.vehicle, self.lowTrailerAttachmentState), nil, connection, self.vehicle);
    end;
end;

function setLowTrailerAttachmentStateEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteInt8(streamId, self.lowTrailerAttachmentState);
end;

function setLowTrailerAttachmentStateEvent.sendEvent(vehicle, lowTrailerAttachmentState, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(setLowTrailerAttachmentStateEvent:new(vehicle, lowTrailerAttachmentState), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(setLowTrailerAttachmentStateEvent:new(vehicle, lowTrailerAttachmentState));
		end;
	end;
end;